searchData={"content_type":"text/markdown","items":[{"doc":"`Guardian.DB` is a simple module that hooks into `Guardian` to prevent\nplayback of tokens.\n\nIn `Guardian`, tokens aren't tracked so the main mechanism that exists to\nmake a token inactive is to set the expiry and wait until it arrives.\n\n`Guardian.DB` takes an active role and stores each token in the database\nverifying it's presence (based on it's jti) when `Guardian` verifies the\ntoken.\nIf the token is not present in the DB, the `Guardian` token cannot be\nverified.\n\nProvides a simple database storage and check for `Guardian` tokens.\n\n- When generating a token, the token is stored in a database.\n- When tokens are verified (channel, session or header) the database is\nchecked for an entry that matches. If none is found, verification results in\nan error.\n- When logout, or revoking the token, the corresponding entry is removed\n\n# Setup\n\n#","ref":"Guardian.DB.html","title":"Guardian.DB","type":"module"},{"doc":"Add your configuration to your environment files. You need to specify\n\n* `repo`\n\nYou may also configure\n\n* `prefix` - The schema prefix to use.\n* `schema_name` - The name of the schema to use. Default \"guardian_tokens\".\n* `sweep_interval` - The interval between db sweeps to remove old tokens.\nDefault 60 (minutes).\n\n#","ref":"Guardian.DB.html#module-config","title":"Config - Guardian.DB","type":"module"},{"doc":"In order to sweep your expired tokens from the db, you'll need to add\n`Guardian.DB.Sweeper` to your supervision tree.\nIn your supervisor add it as a worker\n\n```elixir\nworker(Guardian.DB.Sweeper, [interval: 60])\n```\n\n# Migration\n\n`Guardian.DB` requires a table in your database. Create a migration like the\nfollowing:\n\n```elixir\n  create table(:guardian_tokens, primary_key: false) do\n    add(:jti, :string, primary_key: true)\n    add(:typ, :string)\n    add(:aud, :string)\n    add(:iss, :string)\n    add(:sub, :string)\n    add(:exp, :bigint)\n    add(:jwt, :text)\n    add(:claims, :map)\n    timestamps()\n  end\n```\n\n`Guardian.DB` allow to use a custom schema name when creating the migration.\nYou can configure the schema name from config like the following:\n\n```elixir\nconfig :guardian, Guardian.DB,\n  schema_name: \"my_custom_schema\n```\n\nAnd when you run `mix guardian.db.gen.migration` it'll generate the following\nmigration:\n\n```elixir\n  create table(:my_custom_schema, primary_key: false) do\n    add(:jti, :string, primary_key: true)\n    add(:typ, :string)\n    add(:aud, :string)\n    add(:iss, :string)\n    add(:sub, :string)\n    add(:exp, :bigint)\n    add(:jwt, :text)\n    add(:claims, :map)\n    timestamps()\n  end\n```\n\n`Guardian.DB` works by hooking into the lifecycle of your token module.\n\nYou'll need to add it to\n\n* `after_encode_and_sign`\n* `on_verify`\n* `on_revoke`\n\nFor example:\n\n```elixir\ndefmodule MyApp.AuthTokens do\n  use Guardian, otp_app: :my_app\n\n  # snip...\n\n  def after_encode_and_sign(resource, claims, token, _options) do\n    with {:ok, _} <- Guardian.DB.after_encode_and_sign(resource, claims[\"typ\"], claims, token) do\n      {:ok, token}\n    end\n  end\n\n  def on_verify(claims, token, _options) do\n    with {:ok, _} <- Guardian.DB.on_verify(claims, token) do\n      {:ok, claims}\n    end\n  end\n\n  def on_revoke(claims, token, _options) do\n    with {:ok, _} <- Guardian.DB.on_revoke(claims, token) do\n      {:ok, claims}\n    end\n  end\nend\n```","ref":"Guardian.DB.html#module-sweeper","title":"Sweeper - Guardian.DB","type":"module"},{"doc":"After the JWT is generated, stores the various fields of it in the DB for\ntracking. If the token type does not match the configured types to be stored,\nthe claims are passed through.","ref":"Guardian.DB.html#after_encode_and_sign/4","title":"Guardian.DB.after_encode_and_sign/4","type":"function"},{"doc":"When a token is refreshed, we invalidate the old token and add the new token\nin the DB.","ref":"Guardian.DB.html#on_refresh/2","title":"Guardian.DB.on_refresh/2","type":"function"},{"doc":"When logging out, or revoking a token, removes from the database so the\ntoken may no longer be used.","ref":"Guardian.DB.html#on_revoke/2","title":"Guardian.DB.on_revoke/2","type":"function"},{"doc":"When a token is verified, check to make sure that it is present in the DB.\nIf the token is found, the verification continues, if not an error is\nreturned.\nIf the type of the token does not match the configured token storage types,\nthe claims are passed through.","ref":"Guardian.DB.html#on_verify/2","title":"Guardian.DB.on_verify/2","type":"function"},{"doc":"Revoke all tokens of a given subject. Returns the amount of tokens revoked.","ref":"Guardian.DB.html#revoke_all/1","title":"Guardian.DB.revoke_all/1","type":"function"},{"doc":"Add to your `Guardian` module.\n\n```elixir\ndefmodule MyApp.AuthTokens do\n  use Guardian, otp_app: :my_app\n\n  # snip...\n\n  def revoke_all(resource, claims) do\n    with {:ok, sub} <- subject_for_token(resource, claims) do\n      Guardian.DB.revoke_all(resource)\n    end\n  end\nend\n```\n\nThen you revoke all tokens of a resource.\n\n```elixir\nMyApp.AuthTokens.revoke_all(resource, %{})\n```","ref":"Guardian.DB.html#revoke_all/1-usage","title":"Usage - Guardian.DB.revoke_all/1","type":"function"},{"doc":"The Guardian DB Adapter.\n\nThis behaviour allows to use any storage system\nfor Guardian Tokens.","ref":"Guardian.DB.Adapter.html","title":"Guardian.DB.Adapter","type":"behaviour"},{"doc":"Deletes JWT token\nUsed in `Guardian.DB.Token.destroy_token/3`","ref":"Guardian.DB.Adapter.html#c:delete/2","title":"Guardian.DB.Adapter.delete/2","type":"callback"},{"doc":"Purges all JWT tokens for a given subject.\n\nReturns a tuple containing the number of entries and any returned result as second element.","ref":"Guardian.DB.Adapter.html#c:delete_by_sub/2","title":"Guardian.DB.Adapter.delete_by_sub/2","type":"callback"},{"doc":"Persists JWT token\nUsed in `Guardian.DB.Token.create/2`","ref":"Guardian.DB.Adapter.html#c:insert/2","title":"Guardian.DB.Adapter.insert/2","type":"callback"},{"doc":"Retrieves JWT token\nUsed in `Guardian.DB.Token.find_by_claims/1`","ref":"Guardian.DB.Adapter.html#c:one/2","title":"Guardian.DB.Adapter.one/2","type":"callback"},{"doc":"Purges all expired JWT tokens.\n\nReturns a tuple containing the number of entries and any returned result as second element.","ref":"Guardian.DB.Adapter.html#c:purge_expired_tokens/2","title":"Guardian.DB.Adapter.purge_expired_tokens/2","type":"callback"},{"doc":"Implement the Guardian.DB.Adapter for ETS","ref":"Guardian.DB.ETSAdapter.html","title":"Guardian.DB.ETSAdapter","type":"module"},{"doc":"Implement the Guardian.DB.Adapter for Ecto.Repo","ref":"Guardian.DB.EctoAdapter.html","title":"Guardian.DB.EctoAdapter","type":"module"},{"doc":"A GenServer that periodically checks for, and expires, tokens from storage.\n\nTo leverage the automated Sweeper functionality update your project's Application\nfile to include the following child in your supervision tree:","ref":"Guardian.DB.Sweeper.html","title":"Guardian.DB.Sweeper","type":"module"},{"doc":"worker(Guardian.DB.Sweeper, [interval: 60])","ref":"Guardian.DB.Sweeper.html#module-example","title":"Example - Guardian.DB.Sweeper","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Guardian.DB.Sweeper.html#child_spec/1","title":"Guardian.DB.Sweeper.child_spec/1","type":"function"},{"doc":"Manually trigger a database purge of expired tokens. Also resets the current\nscheduled work.","ref":"Guardian.DB.Sweeper.html#purge/0","title":"Guardian.DB.Sweeper.purge/0","type":"function"},{"doc":"Reset the purge timer.","ref":"Guardian.DB.Sweeper.html#reset_timer/0","title":"Guardian.DB.Sweeper.reset_timer/0","type":"function"},{"doc":"","ref":"Guardian.DB.Sweeper.html#start_link/1","title":"Guardian.DB.Sweeper.start_link/1","type":"function"},{"doc":"A very simple model for storing tokens generated by `Guardian`.","ref":"Guardian.DB.Token.html","title":"Guardian.DB.Token","type":"module"},{"doc":"Create a new token based on the JWT and decoded claims.","ref":"Guardian.DB.Token.html#create/2","title":"Guardian.DB.Token.create/2","type":"function"},{"doc":"Find one token by matching jti and aud.","ref":"Guardian.DB.Token.html#find_by_claims/1","title":"Guardian.DB.Token.find_by_claims/1","type":"function"},{"doc":"Purge any tokens that are expired. This should be done periodically to keep\nyour DB table clean of clutter.","ref":"Guardian.DB.Token.html#purge_expired_tokens/0","title":"Guardian.DB.Token.purge_expired_tokens/0","type":"function"},{"doc":"Generates the required GuardianDb's database migration.\n\nIt allows custom schema name, using the config\nentry `schema_name`.","ref":"Mix.Tasks.Guardian.Db.Gen.Migration.html","title":"mix guardian.db.gen.migration","type":"task"},{"doc":"# Guardian.DB\n\n[![Hex.pm](https://img.shields.io/hexpm/v/guardian_db.svg)](https://hex.pm/packages/guardian_db)\n![Build Status](https://github.com/ueberauth/guardian_db/workflows/Continuous%20Integration/badge.svg)\n[![Codecov](https://codecov.io/gh/ueberauth/guardian_db/branch/master/graph/badge.svg)](https://codecov.io/gh/ueberauth/guardian_db)\n[![Inline docs](https://inch-ci.org/github/ueberauth/guardian_db.svg)](https://inch-ci.org/github/ueberauth/guardian_db)\n\n`Guardian.DB` is an extension to `Guardian` that tracks tokens in your\napplication's database to prevent playback.","ref":"readme.html","title":"Guardian.DB","type":"extras"},{"doc":"`Guardian.DB` assumes that you are using the Guardian framework for\nauthentication.\n\nTo install `Guardian.DB`, first add it to your `mix.exs` file:\n\n```elixir\ndefp deps do\n  [\n    {:guardian_db, \"~> 2.0\"}\n  ]\nend\n```\n\nThen run `mix deps.get` on your terminal.\n\nConfigure your application as seen in the *Configuration* section below prior to attempting to generate the migration or you will get an `application was not loaded/started` error.\n\nFollowing configuration add the Guardian migration:\n\nrun `mix guardian.db.gen.migration` to generate a migration.\n\n**Do not run the migration yet,** we need to complete our setup first.","ref":"readme.html#installation","title":"Installation - Guardian.DB","type":"extras"},{"doc":"```elixir\nconfig :guardian, Guardian.DB,\n  repo: MyApp.Repo, # Add your repository module\n  schema_name: \"guardian_tokens\", # default\n  token_types: [\"refresh_token\"], # store all token types if not set\n  sweep_interval: 60 # default: 60 minutes\n```\n\nTo sweep expired tokens from your db you should add\n`Guardian.DB.Token.SweeperServer` to your supervision tree.\n\n```elixir\nchildren = [\n  {Guardian.DB.Token.SweeperServer, []}\n]\n```\n\n`Guardian.DB` works by hooking into the lifecycle of your `Guardian` module.\n\nYou'll need to add it to:\n\n* `after_encode_and_sign`\n* `on_verify`\n* `on_refresh`\n* `on_revoke`\n\nFor example:\n\n```elixir\ndefmodule MyApp.AuthTokens do\n  use Guardian, otp_app: :my_app\n\n  # snip...\n\n  def after_encode_and_sign(resource, claims, token, _options) do\n    with {:ok, _} <- Guardian.DB.after_encode_and_sign(resource, claims[\"typ\"], claims, token) do\n      {:ok, token}\n    end\n  end\n\n  def on_verify(claims, token, _options) do\n    with {:ok, _} <- Guardian.DB.on_verify(claims, token) do\n      {:ok, claims}\n    end\n  end\n\n  def on_refresh({old_token, old_claims}, {new_token, new_claims}, _options) do\n    with {:ok, _, _} <- Guardian.DB.on_refresh({old_token, old_claims}, {new_token, new_claims}) do\n      {:ok, {old_token, old_claims}, {new_token, new_claims}}\n    end\n  end\n\n  def on_revoke(claims, token, _options) do\n    with {:ok, _} <- Guardian.DB.on_revoke(claims, token) do\n      {:ok, claims}\n    end\n  end\nend\n```\n\nNow run the migration and you'll be good to go.\n\n#","ref":"readme.html#configuration","title":"Configuration - Guardian.DB","type":"extras"},{"doc":"`Guardian.DB` allows custom schema name in migrations, based on following\nconfiguration:\n\n```elixir\nconfig :guardian, Guardian.DB,\n  schema_name: \"my_custom_schema\"\n```\n\nAnd when you run the migration, it'll generate the following migration:\n\n```elixir\n  create table(:my_custom_schema, primary_key: false) do\n    add(:jti, :string, primary_key: true)\n    add(:typ, :string)\n    add(:aud, :string)\n    add(:iss, :string)\n    add(:sub, :string)\n    add(:exp, :bigint)\n    add(:jwt, :text)\n    add(:claims, :map)\n    timestamps()\n  end\n```\n\nThen, run the migration and you'll be good to go.\n\n#","ref":"readme.html#custom-schema-name","title":"Custom schema name - Guardian.DB","type":"extras"},{"doc":"`Guardian` is already a very robust JWT solution. However, if your\napplication needs the ability to immediately revoke and invalidate tokens that\nhave already been generated, you need something like `Guardian.DB` to build upon\n`Guardian`.\n\nIn `Guardian`, you as a systems administrator have no way of revoking\ntokens that have already been generated, you can call `Guardian.revoke`, but in\n`Guardian` **that function does not actually do anything** - it just provides\nhooks for other libraries, such as this one, to define more specific behavior.\nDiscarding the token after something like a log out action is left up to the\nclient application. If the client application does not discard the token, or\ndoes not log out, or the token gets stolen by a malicious script (because the\nclient application stores it in localStorage, for instance), the only thing you\ncan do is wait until the token expires. Depending on the scenario, this may not\nbe acceptable.\n\nWith `Guardian.DB`, records of all generated tokens are kept in your\napplication's database. During each request, the `Guardian.Plug.VerifyHeader`\nand `Guardian.Plug.VerifySession` plugs check the database to make sure the\ntoken is there. If it is not, the server returns a 401 Unauthorized response to\nthe client. Furthermore, `Guardian.revoke!` behavior becomes enhanced, as it\nactually removes the token from the database. This means that if the user logs\nout, or you revoke their token (e.g. after noticing suspicious activity on the\naccount), they will need to re-authenticate.\n\n#","ref":"readme.html#considerations","title":"Considerations - Guardian.DB","type":"extras"},{"doc":"In `Guardian`, token verification is very light-weight. The only thing\n`Guardian` does is decode incoming tokens and make sure they are valid. This can\nmake it much easier to horizontally scale your application, since there is no\nneed to centrally store sessions and make them available to load balancers or\nother servers.\n\nWith `Guardian.DB`, every request requires a trip to the database, as `Guardian`\nnow needs to ensure that a record of the token exists. In large scale\napplications this can be fairly costly, and can arguably eliminate the main\nadvantage of using a JWT authentication solution, which is statelessness.\nFurthermore, session authentication already works this way, and in most cases\nthere isn't a good enough reason to reinvent that wheel using JWTs.\n\nIn other words, once you have reached a point where you think you need\n`Guardian.DB`, it may be time to take a step back and reconsider your whole\napproach to authentication!\n\n#","ref":"readme.html#disadvantages","title":"Disadvantages - Guardian.DB","type":"extras"},{"doc":"We created `Guardian.DB.Adapter` behaviour to allow creating other repositories for persisting JWT tokens. \nYou need to implement the `Guardian.DB.Adapter` behavior working with your preferred storage.     \n\n#","ref":"readme.html#create-your-own-repo","title":"Create your own Repo - Guardian.DB","type":"extras"},{"doc":"1. Redis adapter - [`guardian_redis`](https://github.com/alexfilatov/guardian_redis)\n\nFeel free to create your adapters using `Guardian.DB.Adapter` behavior and you are welcome to add them here.","ref":"readme.html#adapters","title":"Adapters - Guardian.DB","type":"extras"}]}